"""Computer vision toolbox for road infrastructure analysis."""

__version__ = "1.0.4"
__author__ = "Logiroad"
__email__ = "thibaut.deveraux@logiroad-center.com"

# Import main modules for easier access
from . import cvat, encoding, hugging_face, image, placeholders, spatial

__all__ = [
    "__version__",
    "spatial",
    "cvat",
    "image",
    "encoding",
    "hugging_face",
    "placeholders",
]
